@@ -20,19 +20,35 @@ import PIL.Image as Image
 from typing import List, Dict, Tuple
 from PIL import ImageDraw, ImageFont
 from math import cos, sin
+from pybaseutils.coords_utils import *
+from pybaseutils.transforms import affine_transform
 
-color_map = [(0, 0, 0), (128, 0, 0), (0, 128, 0), (128, 128, 0),
+IMG_POSTFIX = ['*.jpg', '*.jpeg', '*.png', '*.tif']
+color_map = [(0, 0, 0), (255, 0, 0), (0, 255, 0), (0, 0, 255),
+             (128, 0, 0), (0, 128, 0), (128, 128, 0),
              (0, 0, 128), (128, 0, 128), (0, 128, 128), (128, 128, 128),
              (64, 0, 0), (192, 0, 0), (64, 128, 0), (192, 128, 0),
              (64, 0, 128), (192, 0, 128), (64, 128, 128), (192, 128, 128),
-             (0, 64, 0), (128, 64, 0), (0, 192, 0), (128, 192, 0), (0, 64, 128)] * 100
+             (0, 64, 0), (128, 64, 0), (0, 192, 0), (128, 192, 0), (0, 64, 128)] * 10
 
 
-def get_colors_color_map(nums):
-    cmap = plt.get_cmap('rainbow')
-    colors = [cmap(i) for i in np.linspace(0, 1, nums + 2)]
-    colors = [(c[2], c[1], c[0]) for c in colors]
-    return colors
+def create_image(shape, color=(255, 255, 255), dtype=np.uint8, uas_rgb=False):
+    """
+    生成一张图片
+    :param shape:
+    :param color: (b,g,r)
+    :param dtype:
+    :param uas_rgb:
+    :return:
+    """
+    image = np.zeros(shape, dtype=np.uint8)
+    ndim = image.ndim
+    if ndim == 2: return np.asarray(image + max(color), dtype=dtype)
+    for i in range(len(color)):
+        image[:, :, i] = color[i]
+    if uas_rgb:
+        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
+    return image
 
 
 def points_protection(points, height, width):
@@ -120,6 +136,12 @@ def untranspose(data):
     return data
 
 
+def swap_image(image):
+    # image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
+    image = image[:, :, ::-1]  # RGB->BGR
+    return image
+
+
 def show_batch_image(title, batch_images, index=0):
     '''
     批量显示图片
@@ -608,7 +630,7 @@ def resize_image(image, size: Tuple[int, int]):
         resize_height = int(height * resize_width / width)
     elif resize_width is None:
         resize_width = int(width * resize_height / height)
-    image = cv2.resize(image, dsize=(resize_width, resize_height))
+    image = cv2.resize(image, dsize=(int(resize_width), int(resize_height)))
     return image
 
 
@@ -828,73 +850,6 @@ def scale_rect(orig_rect, orig_shape, dest_shape):
     return dest_rect
 
 
-def xyxy2xywh(xyxy: np.ndarray):
-    """(xmin,ymin,xmax,ymax)==>(xmin,ymin,w,h)"""
-    xywh = xyxy.copy()
-    xywh[:, 2] = xywh[:, 2] - xywh[:, 0]  # w=xmax-xmin
-    xywh[:, 3] = xywh[:, 3] - xywh[:, 1]  # w=ymax-ymin
-    return xywh
-
-
-def xywh2xyxy(xywh: np.ndarray):
-    """(xmin,ymin,w,h)==>(xmin,ymin,xmax,ymax)"""
-    xyxy = xywh.copy()
-    xyxy[:, 2] = xyxy[:, 0] + xyxy[:, 2]  # xmax=xmin+w
-    xyxy[:, 3] = xyxy[:, 1] + xyxy[:, 3]  # ymax=ymin+h
-    return xyxy
-
-
-def xyxy2cxcywh(xyxy: np.ndarray, width=None, height=None, normalized=False):
-    """(xmin, ymin, xmax, ymax)==>(cx,cy,w,h)"""
-    cxcywh = np.zeros_like(xyxy)
-    cxcywh[:, 0] = (xyxy[:, 2] + xyxy[:, 0]) / 2  # cx
-    cxcywh[:, 1] = (xyxy[:, 3] + xyxy[:, 1]) / 2  # cy
-    cxcywh[:, 2] = (xyxy[:, 2] - xyxy[:, 0])  # w
-    cxcywh[:, 3] = (xyxy[:, 3] - xyxy[:, 1])  # h
-    if normalized:
-        cxcywh = cxcywh / (width, height, width, height)
-    return cxcywh
-
-
-def cxcywh2xyxy(cxcywh: np.ndarray, width=None, height=None, normalized=False):
-    """(cx,cy,w,h)==>xmin, ymin, xmax, ymax)"""
-    xyxy = np.zeros_like(cxcywh)
-    xyxy[:, 0] = cxcywh[:, 0] - cxcywh[:, 2] / 2  # top left x
-    xyxy[:, 1] = cxcywh[:, 1] - cxcywh[:, 3] / 2  # top left y
-    xyxy[:, 2] = cxcywh[:, 0] + cxcywh[:, 2] / 2  # bottom right x
-    xyxy[:, 3] = cxcywh[:, 1] + cxcywh[:, 3] / 2  # bottom right y
-    if normalized:
-        xyxy = xyxy * (width, height, width, height)
-    return xyxy
-
-
-def extend_xyxy(xyxy: np.ndarray, scale=[1.0, 1.0]):
-    """
-    :param bboxes: [[xmin, ymin, xmax, ymax]]
-    :param scale: [sx,sy]==>(W,H)
-    :return:
-    """
-    cxcywh = np.zeros_like(xyxy, dtype=xyxy.dtype)
-    cxcywh[:, 0] = (xyxy[:, 2] + xyxy[:, 0]) / 2  # cx
-    cxcywh[:, 1] = (xyxy[:, 3] + xyxy[:, 1]) / 2  # cy
-    cxcywh[:, 2] = (xyxy[:, 2] - xyxy[:, 0]) * scale[0]  # w
-    cxcywh[:, 3] = (xyxy[:, 3] - xyxy[:, 1]) * scale[1]  # h
-    dxyxy = cxcywh2xyxy(cxcywh, width=None, height=None, normalized=False)
-    return dxyxy
-
-
-def extend_xywh(xywh: np.ndarray, scale=[1.0, 1.0]):
-    """
-    :param bboxes: [[xmin, ymin, xmax, ymax]]
-    :param scale: [sx,sy]==>(W,H)
-    :return:
-    """
-    xyxy = xywh2xyxy(xywh)
-    xyxy = extend_xyxy(xyxy, scale)
-    dxywh = xyxy2xywh(xyxy)
-    return dxywh
-
-
 def get_rect_intersection(rec1, rec2):
     """
     计算两个rect的交集坐标
@@ -1010,7 +965,7 @@ def draw_image_bboxes_labels_text(rgb_image, boxes, labels, boxes_name=None, col
     boxes_name = boxes_name if boxes_name else labels
     for label, box, name in zip(labels, boxes, boxes_name):
         box = [int(b) for b in box]
-        color_ = color if color else color_map[int(label)]
+        color_ = color if color else color_map[int(label) + 1]
         custom_bbox_line(rgb_image, box, color_, str(name), thickness, drawType, top)
     return rgb_image
 
@@ -1102,7 +1057,7 @@ def draw_image_detection_bboxes(rgb_image, bboxes, probs, labels, class_name=Non
     labels = np.asarray(labels, dtype=np.int32).reshape(-1)
     probs = np.asarray(probs).reshape(-1)
     for label, box, prob in zip(labels, bboxes, probs):
-        color = color_map[int(label)]
+        color = color_map[int(label) + 1]
         box = [int(b) for b in box]
         if class_name:
             label = class_name[int(label)]
@@ -1213,18 +1168,16 @@ def show_boxList(title, boxList, rgb_image, delay=0):
     return rgb_image
 
 
-def draw_landmark(image, landmarks_list, color=(0, 0, 255), vis_id=False):
+def draw_landmark(image, landmarks_list, radius=1, thickness=2, color=(0, 0, 255), vis_id=False):
     image = copy.copy(image)
-    point_size = 1
-    thickness = 2  # 可以为 0 、4、8
     for landmarks in landmarks_list:
         for i, landmark in enumerate(landmarks):
             # 要画的点的坐标
             point = (int(landmark[0]), int(landmark[1]))
-            cv2.circle(image, point, point_size, color, thickness)
+            cv2.circle(image, point, radius, color, thickness)
             if vis_id:
-                image = draw_points_text(image, [point], texts=[str(i)], color=color, thickness=thickness,
-                                         drawType="simple")
+                image = draw_points_text(image, [point], texts=[str(i)],
+                                         color=color, thickness=thickness, drawType="simple")
     return image
 
 
@@ -1579,6 +1532,22 @@ def image_rotation(image, angle, center=None, scale=1.0, borderValue=(0, 0, 0)):
     return rotated
 
 
+def image_points_rotation(image, points, angle, center=None, scale=1.0, borderValue=(0, 0, 0)):
+    h, w = image.shape[:2]
+    if not center:
+        # center = (w // 2, h // 2)
+        center = (w / 2., h / 2.)
+    output_size = [w, h]
+    image, points, trans = affine_transform.AffineTransform.affine_transform_for_image_points(image, points,
+                                                                                              output_size,
+                                                                                              center,
+                                                                                              scale=[scale, scale],
+                                                                                              rot=angle,
+                                                                                              inv=False,
+                                                                                              color=borderValue)
+    return image, points
+
+
 def rgb_to_gray(image):
     """
     RGB to Gray image
@@ -1845,14 +1814,14 @@ def get_rect_crop_padding(image, rect, color=(0, 0, 0)):
     return roi_image
 
 
-def get_bbox_crop_padding(image, bbox):
+def get_bbox_crop_padding(image, bbox, color=(0, 0, 0)):
     """
     :param image:
     :param bbox:
     :return:
     """
     rect = bboxes2rects([bbox])[0]
-    roi_image = get_rect_crop_padding(image, rect)
+    roi_image = get_rect_crop_padding(image, rect, color=color)
     return roi_image
 
 
@@ -1880,7 +1849,7 @@ def get_bboxes_crop(image, bboxes):
     return crops
 
 
-def get_bboxes_crop_padding(image, bboxes, size):
+def get_bboxes_crop_padding(image, bboxes, size, color=(0, 0, 0)):
     """
     :param image:
     :param bboxes:
@@ -1890,13 +1859,13 @@ def get_bboxes_crop_padding(image, bboxes, size):
     rects = bboxes2rects(bboxes)
     roi_images = []
     for rect in rects:
-        roi_image = get_rect_crop_padding(image, rect)
+        roi_image = get_rect_crop_padding(image, rect, color=color)
         roi_image = resize_image(roi_image, size=size)
         roi_images.append(roi_image)
     return roi_images
 
 
-def get_rects_crop_padding(image, rects, size):
+def get_rects_crop_padding(image, rects, size, color=(0, 0, 0)):
     """
     :param image:
     :param rects:
@@ -1905,7 +1874,7 @@ def get_rects_crop_padding(image, rects, size):
     """
     roi_images = []
     for rect in rects:
-        roi_image = get_rect_crop_padding(image, rect)
+        roi_image = get_rect_crop_padding(image, rect, color=color)
         roi_image = resize_image(roi_image, size=size)
         roi_images.append(roi_image)
     return roi_images
@@ -2111,7 +2080,7 @@ def boxes2polygons(boxes: np.ndarray or List[np.ndarray]):
     polygons = []
     for box in boxes:
         xmin, ymin, xmax, ymax = box
-        p = [[xmin, ymin], [xmin, ymax], [xmax, ymax], [xmax, ymin]]
+        p = [[xmin, ymin], [xmax, ymin], [xmax, ymax], [xmin, ymax]]
         polygons.append(p)
     polygons = np.asarray(polygons)
     return polygons
@@ -2184,6 +2153,66 @@ def get_mask_iou(mask1, mask2, binarize=True):
     return iou
 
 
+def get_scale_image(image, scale=0.85, offset=(0, 0), color=(0, 0, 0), interpolation=cv2.INTER_NEAREST):
+    """
+    同比居中缩小image，以便居中显示
+    :param image: mask
+    :param scale: 缩放比例
+    :param offset: 偏移量(x,y)
+    :return: 返回缩放的轮廓Mask
+    """
+    h, w = image.shape[:2]
+    bg = create_image(image.shape, color=color)
+    fg = cv2.resize(image, dsize=(int(w * scale), int(h * scale)), interpolation=interpolation)
+    sh, sw = fg.shape[:2]
+    xmin = (w - sw) // 2 + offset[0]
+    ymin = (h - sh) // 2 + offset[1]
+    bg[ymin:ymin + sh, xmin:xmin + sw] = fg
+    return bg
+
+
+get_scale_mask = get_scale_image
+
+
+def get_scale_contours(contours, size, scale=0.85, offset=(0, 0)):
+    """
+    同比居中缩小mask的轮廓，以便居中显示
+    :param contours:  mask的轮廓
+    :param size: 输出大小(W,H)
+    :param scale: 缩放比例
+    :param offset: 偏移量(x,y)
+    :return: 返回缩放的轮廓contours
+    """
+    dst_contours = copy.deepcopy(contours)
+    sw, sh = (int(size[0] * scale), int(size[1] * scale))
+    xmin = (size[0] - sw) // 2
+    ymin = (size[1] - sh) // 2
+    for i in range(len(dst_contours)):
+        for c in range(len(dst_contours[i])):
+            d = dst_contours[i][c] * scale + (xmin, ymin) + offset
+            dst_contours[i][c] = np.asarray(d, dtype=np.int32)
+    return dst_contours
+
+
+def resize_image_points(image: np.ndarray, points, size):
+    """
+    缩放图片，并缩放对应的points
+    :param image: 图片
+    :param points: shape is (N,2)
+    :param size:
+    :return:
+    """
+    height, width = image.shape[:2]
+    image = resize_image(image, size=size)
+    resize_height, resize_width = image.shape[:2]
+    scale = [resize_width / width, resize_height / height]
+    if len(points) > 0:
+        if points.shape[1] > 2:  # 如果points是多维度
+            scale += [0] * (points.shape[1] - 2)
+        points = np.asarray(points * scale)
+    return image, points
+
+
 def pointPolygonTest(point, contour, measureDist=False):
     """
     此功能可查找图像中的点与轮廓之间的最短距离.
@@ -2225,6 +2254,7 @@ def get_mask_boundrect_cv(mask, binarize=False, shift=0):
     if binarize:
         ret, mask = cv2.threshold(mask, 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)
     contours = find_mask_contours(mask)
+    if len(contours) == 0: return []
     contours = np.concatenate(contours)
     xmin = np.min(contours[:, 0])
     ymin = np.min(contours[:, 1])
@@ -2312,6 +2342,45 @@ def find_image_contours(mask: np.ndarray, target_label: List[int] = [1, 2]) -> L
     return contours
 
 
+def get_image_points_valid_range(image, points, valid_range, crop=True, color=(255, 255, 255)):
+    """
+    获得在valid_range范围的points
+    :param image: 图像
+    :param points: 点集合shape: (N,2)
+    :param valid_range: 有效范围(xmin,ymin,xmax,ymax)
+    :param crop: 是否采裁剪图像在valid_range有效区域
+    :param color: crop图像时，填充颜色
+    :return:
+    """
+    valid_point, valid_index = get_points_valid_range(points, valid_range)
+    if crop:
+        image = get_bbox_crop_padding(image, valid_range, color=color)
+        valid_point = valid_point - (valid_range[0], valid_range[1])
+    return image, valid_point, valid_index
+
+
+def get_points_valid_range(points, valid_range):
+    """
+    获得在valid_range范围的points
+    :param points: 点集合shape: (N,2)
+    :param valid_range: 有效范围(xmin,ymin,xmax,ymax)
+    :return:
+    """
+    xmin, ymin, xmax, ymax = valid_range
+    contour = [[xmin, ymin], [xmin, ymax], [xmax, ymax], [xmax, ymin]]
+    valid_point = []
+    valid_index = []
+    for i in range(len(points)):
+        pt = points[i, :]
+        dist = pointPolygonTest(pt, contour)
+        if dist > 0:
+            valid_point.append(pt)
+            valid_index.append(i)
+    valid_point = np.asarray(valid_point)
+    valid_index = np.asarray(valid_index)
+    return valid_point, valid_index
+
+
 def fig2data(fig):
     """
     fig = plt.figure()
@@ -2440,24 +2509,26 @@ def image_composite(image: np.ndarray, alpha: np.ndarray, bg_img=(219, 142, 67))
     return image
 
 
-def frames2gif_by_imageio(frames, gif_file="test.gif", fps=2, loop=0):
+def frames2gif_by_imageio(frames, gif_file="test.gif", fps=2, loop=0, use_rgb=False):
     """
     pip install imageio
     :param frames:
     :param gif_file: 输出的GIF图的路径
     :param fps: 刷新频率
     :param loop: 循环次数
+    :param use_rgb: frames是RGB格式，需要是BGR格式，use_bgr=True
     :return:
     """
     import imageio
     writer = imageio.get_writer(uri=gif_file, mode='I', fps=fps, loop=loop)
     for image in frames:
+        if use_rgb: image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
         writer.append_data(image)
     writer.close()
     # imageio.mimwrite(out_gif_path, frames, fps=20)
 
 
-def frames2gif_by_pil(frames, gif_file="test.gif", fps=2, loop=0):
+def frames2gif_by_pil(frames, gif_file="test.gif", fps=2, loop=0, use_rgb=False):
     """
     https://pillow.readthedocs.io/en/stable/handbook/image-file-formats.html#gif
     PS :loop=0表示无限循环，loop=n表示循环n+1次，如要循环一次，需要去掉loop参数(很奇葩吧！！！)
@@ -2467,6 +2538,7 @@ def frames2gif_by_pil(frames, gif_file="test.gif", fps=2, loop=0):
     :param loop: 循环次数
     :return:
     """
+    if use_rgb: frames = [cv2.cvtColor(img, cv2.COLOR_BGR2RGB) for img in frames]
     images = [Image.fromarray(img) for img in frames]
     if loop == 1:
         images[0].save(gif_file, save_all=True, append_images=images[1:], duration=1000 / fps,
@@ -2591,7 +2663,7 @@ class CVVideo():
 
 
 if __name__ == "__main__":
-    from utils import image_utils
+    # from utils import image_utils
 
     input_size = [int(490 / 2), int(800 / 2)]
     image_path = "test.jpg"
