@@ -155,8 +155,25 @@ def find_key_metadata(metadata: List[Dict], key):
     return values
 
 
-def show_word_packer(packer, image, keys=[], split_line=False, color=(0, 0, 255), thickness=2, delay=0):
+def draw_word_packer_in_image(packer, image, vis=False, delay=0):
+    for word in packer:
+        image = draw_word_in_image(image, word["mask"], word["box"])
+    if vis: image_utils.cv_show_image("image-mask", image, delay=delay)
+    return image
+
+
+def draw_word_in_image(image, mask, box):
+    if not isinstance(image, np.ndarray): return image
+    xmin, ymin, xmax, ymax = box
+    mask: np.ndarray = image_utils.resize_image(mask, size=(xmax - xmin, ymax - ymin))
+    if mask.ndim == 2: mask = cv2.cvtColor(mask, cv2.COLOR_GRAY2BGR)
+    image = image_utils.cv_paste_image(image, mask, start_point=(int(xmin), int(ymin)))
+    return image
+
+
+def show_word_packer(packer, image, keys=[], split_line=False, color=(255, 0, 0), thickness=2, vis=False, delay=0):
     _keys = ['label', 'cls_score', 'box', 'det_score']
+    mask = image.copy() if isinstance(image, np.ndarray) else None
     for word in packer:
         label = word['label'] if "label" in word else ""
         images = [word[k] for k in keys if k in word]
@@ -168,13 +185,15 @@ def show_word_packer(packer, image, keys=[], split_line=False, color=(0, 0, 255)
                                                        boxes_name=[label],
                                                        color=color,
                                                        thickness=thickness)
-            show_images("dets", [image], split_line=split_line, delay=1)
-        show_images("packer", images, split_line=split_line, delay=delay)
-    return image
+            mask = draw_word_in_image(mask, word["mask"], word["box"])
+            if vis: show_images("dets", [image, mask], split_line=split_line, delay=1)
+        if vis: show_images("packer", images, split_line=split_line, delay=delay)
+    return image, mask
 
 
-def show_word_unpacker(unpacker, image, keys=[], split_line=False, color=(0, 0, 255), thickness=2, delay=0):
+def show_word_unpacker(unpacker, image, keys=[], split_line=False, color=(255, 0, 0), thickness=2, vis=False, delay=0):
     _keys = ['label', 'cls_score', 'box', 'det_score']
+    mask = image.copy() if isinstance(image, np.ndarray) else None
     for i in range(len(unpacker["box"])):
         label = unpacker["label"][i] if "label" in unpacker else ""
         images = [unpacker[k][i] for k in keys if k in unpacker]
@@ -186,9 +205,10 @@ def show_word_unpacker(unpacker, image, keys=[], split_line=False, color=(0, 0,
                                                        boxes_name=[label],
                                                        color=color,
                                                        thickness=thickness)
-            show_images("dets", [image], split_line=split_line, delay=1)
-        show_images("unpacker", images, split_line=split_line, delay=delay)
-    return image
+            mask = draw_word_in_image(mask, unpacker["mask"][i], unpacker["box"][i])
+            if vis: show_images("dets", [image, mask], split_line=split_line, delay=1)
+        if vis: show_images("unpacker", images, split_line=split_line, delay=delay)
+    return image, mask
 
 
 def show_images(title, images, use_rgb=False, split_line=False, delay=0):
@@ -214,3 +234,54 @@ def save_packer_images(packer, keys=["crop"], dsize=None, out_dir="./output"):
             file = file_utils.create_dir(out_dir, None, name)
             img = image_utils.resize_image(word[k], size=dsize)
             cv2.imwrite(file, img)
+
+
+def show_word_grid(packer, image, thickness=5, vis=True, delay=0):
+    for i, word in enumerate(packer):
+        color = image_utils.color_map[i + 1]
+        boxes = [word['box']]
+        grid_point = word['grid_point']
+        thickness_ = max(int(thickness * 0.5), 1)
+        image = image_utils.draw_image_boxes(image, boxes, color=color, thickness=thickness_)
+        image = image_utils.draw_image_lines(image, grid_point, thickness=thickness, color=color)
+        if vis: image_utils.cv_show_image("image", image, use_rgb=False, delay=delay)
+
+
+def get_grid_range(mask, word_box, grid_box):
+    """
+    获得网格有效区域
+    :param mask: 文字mask
+    :param word_box: 文字区域框 (xmin,ymin,xmax,ymax)
+    :param grid_box: 网格区域框 (xmin,ymin,xmax,ymax)
+    :return: grid_range: 网格有效区域 (x,y,w,h)
+    """
+    if sum(grid_box) < 0: return []
+    h, w = mask.shape[:2]
+    rect = image_utils.xyxy2xywh([word_box, grid_box])
+    scale = (w, h) / rect[0:1, 2:4]
+    scale = np.repeat(scale, 2, axis=-1)[:, [0, 2, 1, 3]]
+    rect = np.asarray(rect * scale, dtype=np.int32)
+    word_rect, grid_rect = rect[0, :], rect[1, :]
+    grid_range = [grid_rect[0] - word_rect[0], grid_rect[1] - word_rect[1], word_rect[2], word_rect[3]]
+    return grid_range
+
+
+def get_grid_rang_mask(mask: np.ndarray or List, grid_range):
+    """
+    根据网格区域对mask进行填充
+    Usage:
+        from pybaseutils import word_utils
+        grid_range = word_utils.get_grid_range(mask, hw_word["word_box"], hw_word["grid_box"])      # 获得网格有效区域
+        grid_mask = word_utils.get_grid_rang_mask(mask, grid_range=grid_range)                      # 获得网格区域的mask
+        stroke_segs = word_utils.get_grid_rang_mask(hw_word["stroke_segs"], grid_range=grid_range)  # 获得网格区域的笔画mask
+        piece_segs = word_utils.get_grid_rang_mask(hw_word["piece_segs"], grid_range=grid_range)    # 获得网格区域的笔段mask
+    :param mask: 文字mask或者mask列表
+    :param grid_range: 网格有效区域 (x,y,w,h)，可由get_grid_range计算获得
+    :return: mask
+    """
+    if len(grid_range) == 0: return mask
+    if isinstance(mask, np.ndarray):
+        mask = image_utils.get_rect_crop_padding(mask, grid_range, color=(0, 0, 0))
+    elif isinstance(mask, List):
+        mask = [get_grid_rang_mask(m, grid_range) for m in mask]
+    return mask
